local _G = _G
local tonumber = _G.tonumber
local ipairs = _G.ipairs
local select = _G.select
local GetTalentInfo = _G.GetTalentInfo
local AceLibrary = _G.AceLibrary

local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 38334 $"):match("%d+"))

if _G.ThreatLib_OUT_OF_DATE or not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	_G.ThreatLib_OUT_OF_DATE = true
	return
end

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local _, c = _G.UnitClass("player")
if c ~= "PRIEST" then return end 

local ThreatLib = _G.ThreatLib

local DiscSpellsEN = {
	"Consume Magic", "Starshards", "Dispel Magic", "Divine Spirit", "Elune's Grace", "Inner Fire", "Levitate", "Mass Dispel", 
	"Power Word: Fortitude", "Power Word: Shield", "Prayer of Fortitude", "Prayer of Spirit", "Shackle Undead", "Symbol of Hope",
	"Feedback", "Mana Burn"
}

local HolySpellsEn = {
	"Abolish Disease", "Circle of Healing", "Cure Disease", "Desperate Prayer", "Fear Ward", "Flash Heal", "Greater Heal", "Heal", "Holy Fire",
	"Lesser Heal", "Lightwell", "Prayer of Healing", "Prayer of Mending", "Renew", "Resurrection", "Smite"
}

local ShadowSpellsEN = {
	"Prayer of Shadow Protection", "Devouring Plague", "Fade", "Hex of Weakness", "Mind Blast", "Mind Control", "Mind Flay", "Mind Soothe", "Mind Vision", "Psychic Scream",
	"Shadow Protection", "Shadow Word: Death", "Shadow Word: Pain", "Shadowfiend", "Shadowguard", "Touch of Weakness", "Vampiric Touch"
}

local BS = AceLibrary("Babble-Spell-2.2")
ThreatLib:GetModule("ClassCore"):SetModuleDefaultState(c, false)
local Priest = ThreatLib:GetModule("ClassCore"):NewModule(c)
local Aura = AceLibrary("SpecialEvents-Aura-2.0")

local threatAmounts = {
	["reflectiveshield"] = 0,
	["fade"] = {55, 155, 285, 440, 620, 820, 1500}
}

function Priest:ClassInit()	
	self.className = "PRIEST"
	self.passiveThreatModifiers = 1
	
	local function modForSilentResolve(self, amt) return amt * self.silentResolveMod end
	local function modForShadowAffinity(self, amt) return amt * self.shadowAffinityMod end
	
	for _,v in ipairs(DiscSpellsEN) do
		self.abilityThreatMods[BS[v]] = modForSilentResolve
	end
	DiscSpellsEN = nil

	for _,v in ipairs(HolySpellsEn) do
		self.abilityThreatMods[BS[v]] = modForSilentResolve
	end
	HolySpellsEn = nil

	for _,v in ipairs(ShadowSpellsEN) do
		self.abilityThreatMods[BS[v]] = modForShadowAffinity
	end
	ShadowSpellsEN = nil
	
	self.fadedAmount = 0
	
	-- CastBuff handlers should take a multiplier and modify it by any multipliers they infer
	self.ClassBuffs[BS["Fade"]] = self.Fade

	-- ThreatMods should return the modifications (if any) to the threat passed in
	self.abilityThreatMods[BS["Holy Nova"]] = function(amt) return 0 end
	self.abilityThreatMods[BS["Binding Heal"]] = function(amt) return amt * 0.5 * self.silentResolveMod end
end

function Priest:ScanTalents()
	-- Scan talents
	self.silentResolveMod = 1 - (0.04 * select(5, GetTalentInfo(1, 3)))
	self.shadowAffinityMod = 1 - (0.08 * select(5, GetTalentInfo(3, 3)))
end

function Priest:Fade(action, rank, applications)
	if action == "lose" then
		self:_addThreat(self.fadedAmount)
	elseif action == "gain" then
		local ranknum = tonumber(rank:match("%d+"))
		self.fadedAmount = max(0, threatAmounts["fade"][ranknum] * self.shadowAffinityMod)
		self:_addThreat(self.fadedAmount * -1)
	end
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Priest = ThreatLib:GetModule("ClassCore"):GetModule(c)
end)
