local _G = _G
local tonumber = _G.tonumber
local ipairs = _G.ipairs
local select = _G.select
local GetTalentInfo = _G.GetTalentInfo
local AceLibrary = _G.AceLibrary

local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 37315 $"):match("%d+"))

if _G.ThreatLib_OUT_OF_DATE or not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	_G.ThreatLib_OUT_OF_DATE = true
	return
end

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local _, c = _G.UnitClass("player")
if c ~= "SHAMAN" then return end 

local ThreatLib = _G.ThreatLib

local BS = AceLibrary("Babble-Spell-2.2")
ThreatLib:GetModule("ClassCore"):SetModuleDefaultState(c, false)
local Shaman = ThreatLib:GetModule("ClassCore"):NewModule(c)

local healingSpells = { "Lesser Healing Wave", "Healing Wave", "Chain Heal", "Earth Shield", "Healing Stream Totem" }
local damageSpells = { "Lightning Shield", "Chain Lightning", "Lightning Bolt", "Earth Shock", "Flame Shock" }

function Shaman:ClassInit()	
	self.className = "SHAMAN"
	self.passiveThreatModifiers = 1
	
	-- CastHandlers should return true if they modified the threat level, false if they don't
	--self.CastHandlers[BS["Another Example"]] = self.AnotherExample
	
	-- ThreatMods should return the modifications (if any) to the threat passed in
	for _,v in ipairs(healingSpells) do
		self.abilityThreatMods[BS[v]] = self.HealingSpells
	end
	healingSpells = nil

	for _,v in ipairs(damageSpells) do
		self.abilityThreatMods[BS[v]] = self.DamageSpells
	end
	damageSpells = nil

	self.abilityThreatMods[BS["Frost Shock"]] = self.FrostShock
	--self.abilityThreatMods[BS["Stormstrike"]] = self.Melee
	
	self.schoolThreatMods["Physical"] = self.Melee

	self.threatValues = {
		["frostshock"] = 2,
	}	
end

function Shaman:ScanTalents()
	-- Scan talents
	self.HealingGrace = 1 - 0.05 * select(5, GetTalentInfo(3,9))
	self.ElementalPrecision = 1 - 0.04 * select(5, GetTalentInfo(1,15))
	self.SpiritWeapons = 1 - 0.15 * select(5, GetTalentInfo(2,13))
end

function Shaman:HealingSpells(amt)
	return amt * self.HealingGrace
end

function Shaman:DamageSpells(amt)
	return amt * self.ElementalPrecision
end

function Shaman:FrostShock(amt)
	return amt * self.threatValues.frostshock * self.ElementalPrecision
end

function Shaman:Melee(amt)
	return amt * self.SpiritWeapons
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Shaman = ThreatLib:GetModule("ClassCore"):GetModule(c)
end)
