local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 37586 $"):match("%d+"))

if _G.ThreatLib_OUT_OF_DATE or not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	_G.ThreatLib_OUT_OF_DATE = true
	return
end

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local ThreatLib = _G.ThreatLib

local BB = AceLibrary("Babble-Boss-2.2")

-- You need to register the module with the localized mob name, as this is used to 
local c = BB["Leotheras the Blind"]

ThreatLib:GetModule("NPCCore"):SetModuleDefaultState(c, false)
local Leotheras = ThreatLib:GetModule("NPCCore"):NewModule(c)

local demonTriggerStrings = {
	["enUS"] = "I am in control now!$",
	["deDE"] = "Ich habe jetzt die Kontrolle!",
	["frFR"] = "Je prend le contrle",
	["koKR"] = "???, ?? ???. ????? ?? ????!$",
	["zhTW"] = nil
}

local imageTriggerStrings = {
	["enUS"] = "I am the master! Do you hear?",
	["deDE"] = "Ich bin der Meister! H\195\182rt Ihr?",
	["frFR"] = "C'est moi le matre! Vous entendez?",
	["koKR"] = "??? ???? ??? ?? ??...",
	["zhTW"] = nil
}

local initTriggerStrings = {
	["enUS"] = "Finally, my banishment ends!",
	["deDE"] = "Endlich hat meine Verbannung ein Ende!",
	["frFR"] = "Enfin, mon exil s'achve!",
	["koKR"] = "???, ?? ?????!",
	["zhTW"] = nil
}

local whirlwindTriggerStrings = {
	["enUS"] = "Leotheras the Blind gains Whirlwind",
	["deDE"] = "Leotheras der Blinde bekommt Wirbelwind",
	["frFR"] = "Leotheras l'Aveugle gagne Tourbillon",
	["koKR"] = "??? ?????|1?;?; ???? ??? ?????.",
	["zhTW"] = nil
}

local whirlwindTrigger = whirlwindTriggerStrings[_G.GetLocale()]

function Leotheras:Init()	
	local demonPhase = demonTriggerStrings[_G.GetLocale()]
	local splitPhase = imageTriggerStrings[_G.GetLocale()]
	local initialPhase = initTriggerStrings[_G.GetLocale()]
	
	self:RegisterCombatant(BB["Leotheras the Blind"], true)
	self:RegisterCombatant(BB["Shade of Leotheras"], false)

	-- To register a callback to a yell or emote:
	-- self:RegisterChatEvent([yell|event], localizedMobName, yellOrEmoteToTriggerOn, callbackFunction)
	
	self:RegisterChatEvent("yell", BB["Leotheras the Blind"], demonPhase, self.phaseTransition)
	self:RegisterChatEvent("yell", BB["Leotheras the Blind"], splitPhase, self.splitTransition)
	self:RegisterChatEvent("yell", BB["Shade of Leotheras"], splitPhase, self.splitTransition)		--I'm not sure if the shade or leo himself does the yell
	self:RegisterChatEvent("yell", BB["Leotheras the Blind"], initialPhase, self.phaseTransition)
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS")
end

function Leotheras:phaseTransition()
	self:SetNumberOfMobs(1)
	self:WipeRaidThreat("Leotheras")
end

function Leotheras:StartFight()
	self:SetNumberOfMobs(1)
end

function Leotheras:splitTransition()
	self:SetNumberOfMobs(2)		-- Leotheras + The shade of leotheras
	self:WipeRaidThreat("Leotheras")
end

function Leotheras:CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS(msg)
	if msg:find(whirlwindTrigger) then
		self:WipeRaidThreat("Leotheras")
	end
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Leotheras = ThreatLib:GetModule("NPCCore"):GetModule(c)
end)
