local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 38331 $"):match("%d+"))

local useParser30 = AceLibrary:HasInstance("Parser-3.0")

if _G.ThreatLib_OUT_OF_DATE or not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	_G.ThreatLib_OUT_OF_DATE = true
	return
end

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local GenericAttackTranslations = {
	["Knock Away"] = "Knock Away",
	["Wing Buffet"] = "Wing Buffet",
	["Sand Blast"] = "Sand Blast"
}
if GetLocale() == "deDE" then
	-- alter GenericAttackTranslations
end

local new, del, newHash, newSet = ThreatLib.new, ThreatLib.del, ThreatLib.newHash, ThreatLib.newSet

local ThreatLib = _G.ThreatLib
local ThreatLibNPCModuleCore = nil
if useParser30 then
	ThreatLibNPCModuleCore = ThreatLib:NewModule("NPCCore", "AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0", "AceModuleCore-2.0", "Parser-3.0")
	ThreatLibNPCModuleCore:SetModuleMixins("AceEvent-2.0", "AceConsole-2.0", "Parser-3.0")
else
	ThreatLibNPCModuleCore = ThreatLib:NewModule("NPCCore", "AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0", "AceModuleCore-2.0")
	ThreatLibNPCModuleCore:SetModuleMixins("AceEvent-2.0", "AceConsole-2.0")
end

local combatantModuleLookup = {}

local function ModifyThreat(mob, target, multi, add)
	local module
	if target == "player" then
		module = ThreatLib:GetPlayerModule()
	elseif target == "pet" and ThreatLib:DoesPlayerHavePet() then
		module = ThreatLib:GetPetModule()
	end
	if module then
		module:_multiplyTargetThreat(multi, mob)
		module:_addTargetThreat(add, mob)
	end
end

function ThreatLibNPCModuleCore:OnInitialize()
	self.activeModule = nil
	self.generic_attacks = {}
	self.generic_attacks[GenericAttackTranslations["Knock Away"]] = function(mob, target) ModifyThreat(mob, target, 0.5, 0) end
	self.generic_attacks[GenericAttackTranslations["Wing Buffet"]] = function(mob, target) ModifyThreat(mob, target, 0.5, 0) end
	self.generic_attacks[GenericAttackTranslations["Sand Blast"]] = function(mob, target) ModifyThreat(mob, target, 0, 0) end		
	if useParser30 then
		self:RegisterParserEvent({
			eventType = "Damage",
			recipientID = "player"
		}, "parseMobAttack")
		self:RegisterParserEvent({
			eventType = "Damage",
			recipientID = "pet"
		}, "parseMobAttack")	
		self:RegisterParserEvent({
			eventType = "Death"
		}, "parseDeath")
	else
		-- TODO: Parser-1.1 compat
	end
	
	self.encounterMobCount = 1
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
end

function ThreatLibNPCModuleCore:parseMobAttack(info)
	if self.activeModule and self.activeModule:parseMobAttack(info) then
		return
	end

	if self.generic_attacks[info.abilityName] then
		self.generic_attacks[info.abilityName](info.sourceName, info.recipientID)
	end	
end

function ThreatLibNPCModuleCore:parseDeath(info)
	if self.activeModule then
		self.activeModule:parseDeath(info)
	end
end

function ThreatLibNPCModuleCore:UPDATE_MOUSEOVER_UNIT()
	if not UnitExists("mouseover") or UnitIsDead("mouseover") then
		return
	end
	local mobName = UnitName("mouseover")
	if combatantModuleLookup[mobName] and self.activeModule ~= combatantModuleLookup[mobName] then
		ThreatLibNPCModuleCore:ToggleModuleActive(combatantModuleLookup[mobName], true)
	end
end

function ThreatLibNPCModuleCore.modulePrototype:OnInitialize()
	self.chatEvents = {}
	self.encounterEnemies = {}
	self.attacks = {}
	self.encounterMobCount = 1
	self:Init()
	self.ModifyThreat = function(mob, player, multi, add) ModifyThreat(mob, player, multi, add) end 
end

function ThreatLibNPCModuleCore.modulePrototype:OnEnable()
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
	
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	
	-- Replace any active module with ourselves after a clean shutdown
	if ThreatLibNPCModuleCore.activeModule then
		ThreatLibNPCModuleCore:ToggleModuleActive(ThreatLibNPCModuleCore.activeModule, false)
	end
	ThreatLibNPCModuleCore.activeModule = self
end

function ThreatLibNPCModuleCore.modulePrototype:OnDisable()
	ThreatLibNPCModuleCore.activeModule = nil
end

function ThreatLibNPCModuleCore.modulePrototype:parseDeath(info)
	if not ThreatLib.inCombat then
		return
	end
	local recipientName = info.recipientName
	local func = self.encounterEnemies[recipientName]
	if func then
		func(self, recipientName, info.sourceName)
		self:TriggerEvent("ThreatLib_BossDeath", recipientName)
	end
end

function ThreatLibNPCModuleCore.modulePrototype:parseMobAttack(info)
	if not ThreatLib.inCombat then
		return
	end
	local func = self.attacks[info.abilityName]
	if func then
		func(self, info.sourceName, info.recipientID)
		return true
	end
end

function ThreatLibNPCModuleCore.modulePrototype:OnDisable()
	--[[
	for evtType, mobs in pairs(self.chatEvents) do
		for mob, texts in pairs(mobs) do
			self.chatEvents[evtType][mob] = del(self.chatEvents[evtType][mob])
		end
		self.chatEvents[evtType] = del(self.chatEvents[evtType])
	end
	]]--
	ThreatLibNPCModuleCore.activeModule = nil
end

function ThreatLibNPCModuleCore.modulePrototype:CHAT_MSG_MONSTER_EMOTE(arg1, arg2)
	local emote = self.chatEvents.emote
	if not emote then
		return
	end
	local mob = emote[arg2]
	if not mob then
		return
	end
	local func = mob[arg1]
	if not func then
		return
	end
	func(self)
end

function ThreatLibNPCModuleCore.modulePrototype:CHAT_MSG_MONSTER_YELL(arg1, arg2)
	local yell = self.chatEvents.yell
	if not yell then
		return
	end
	local yell_arg2 = yell[arg2]
	if not yell_arg2 then
		return
	end
	yell_arg2(self)
end

function ThreatLibNPCModuleCore.modulePrototype:PLAYER_REGEN_ENABLED()
	self:ScheduleEvent("NPCModuleCoreResetMobCount" .. tostring(self), "ResetFight", 5)
end

function ThreatLibNPCModuleCore.modulePrototype:PLAYER_REGEN_DISABLED()
	if self:IsEventScheduled("NPCModuleCoreResetMobCount" .. tostring(self)) then
		self:CancelScheduledEvent("NPCModuleCoreResetMobCount" .. tostring(self))
	end
end

function ThreatLibNPCModuleCore.modulePrototype:RegisterCombatant(combatantName, callback)
	combatantModuleLookup[combatantName] = self
	
	if not callback then
		return
	end
	if callback == true then
		callback = self.BossDeath
	end
	self.encounterEnemies[combatantName] = callback
	
	if useParser30 then
		self:RegisterParserEvent({
			eventType = "Damage",
			recipientName = combatantName
		}, "StartCombat")	

		self:RegisterParserEvent({
			eventType = "Cast",
			recipientName = combatantName
		}, "StartCombat")	
	else
		-- TODO: Parser-1.1 implementation
	end
end

-- Override to implement state reset on fight start
function ThreatLibNPCModuleCore.modulePrototype:StartFight()
end

function ThreatLibNPCModuleCore.modulePrototype:SetNumberOfMobs(num)
	self.encounterMobCount = max(1, num)
	ThreatLibNPCModuleCore.encounterMobCount = self.encounterMobCount
end

function ThreatLibNPCModuleCore.modulePrototype:GetNumberOfMobs()
	return self.encounterMobCount
end

function ThreatLibNPCModuleCore.modulePrototype:ResetFight()
	self:SetNumberOfMobs(1)
end

function ThreatLibNPCModuleCore.modulePrototype:StartCombat()
	if not ThreatLib.inCombat then
		self:StartFight()
	end
end

function ThreatLibNPCModuleCore.modulePrototype:BossDeath()
	self:ResetFight()
	ThreatLibNPCModuleCore:ToggleModuleActive(self, false)
end

local new, del = ThreatLib.new, ThreatLib.del

function ThreatLibNPCModuleCore.modulePrototype:RegisterChatEvent(eventType, mob, text, callback)
	if not mob then
		return
	end
	local t = self.chatEvents[eventType]
	if not t then
		t = new()
		self.chatEvents[eventType] = t
	end
	
	local tm = t[mob]
	if not tm then
		tm = new()
		t[mob] = tm
	end
	tm[text] = callback		
end

function ThreatLibNPCModuleCore.modulePrototype:UnregisterChatEvent(eventType, mob, text)
	local t = self.chatEvents[eventType]
	if not t then
		return
	end
	
	local tm = t[mob]
	if not tm then
		return
	end
	
	tm[text] = nil
	if not next(tm) then
		t[mob] = del(tm)
	end
	if not next(t) then
		self.chatEvents[eventType] = del(t)
	end
end

----------------------------------------------------------------
-- Module API
----------------------------------------------------------------

function ThreatLibNPCModuleCore.modulePrototype:MultiplyRaidThreatForMob(mob, amount)
	ThreatLib:SetRaidThreatMultiplier(mob, amount)
end

function ThreatLibNPCModuleCore.modulePrototype:WipeRaidThreat()
	ThreatLib:SetRaidThreatMultiplier(tostring(self), 0)
end

function ThreatLibNPCModuleCore.modulePrototype:WipeRaidThreatOnMob(mob)
	ThreatLib:WipeRaidThreatOnMob(mob)
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	ThreatLibNPCModuleCore = ThreatLib:GetModule("NPCCore")
end)
